
function HTTPRequest(url){
	this.urlParts = this.parseUrl(url);
	this.socket = null;
	this.buffer = '';
	this.socketIsEOF=false;
}

HTTPRequest.prototype.parseUrl = function(url){
	var parts = {};

	//All I need is the domain and the path.
	var pos = url.indexOf('://');
	if (pos != -1){
		url = url.substring(pos+3);
	}

	pos = url.indexOf('/');
	if (pos != -1){
		parts.domain = url.substring(0, pos);
		parts.path = url.substring(pos);
	}
	else {
		parts.domain = url;
		parts.path = '/';
	}


	pos = parts.domain.indexOf(':');
	if (pos != -1){
		parts.port = parseInt(parts.domain.substring(pos+1));
		parts.domain = parts.domain.substring(0, pos);
	}
	else {
		parts.port = 80;
	}
	return parts;
};

HTTPRequest.prototype.open = function(method, postData){
	this.socket = new Socket();
	this.socket.connect({family:Socket.PF_INET, details:{address:this.urlParts.domain, port:this.urlParts.port}});

	var headers = method+' '+this.urlParts.path+' HTTP/1.0';
	headers += '\r\nHost: '+this.urlParts.domain;
	headers += '\r\nConnection: close';

	if (method.toUpperCase() == 'POST'){
		headers += '\r\nContent-length: '+postData.length;
		headers += '\r\nContent-type: application/x-www-formdata';
		headers += '\r\n\r\n';
		headers += postData;
	}
	else {
		headers += '\r\n\r\n';
	}

	this.socket.write(headers);
};

HTTPRequest.prototype.downloadToFile = function(file){
	var fileInfo = new File(file);
	var doneHeaders = false;
	if (!fileInfo.exists())
	{
		fileInfo.create();
	}
	var file = new FileStream(file);
	
	file.open();
	while ((data=this.read()).length > 0){
		if (!doneHeaders && data.trim() == ''){
			doneHeaders = true;
		}
		else if (doneHeaders){
			file.write(data);
		}
	}

	file.close();
}

HTTPRequest.prototype.download = function(){
	var data = '';
	var buffer = '';
	var doneHeaders = false;

	while ((buffer = this.read()).length > 0){
		if (!doneHeaders && buffer.trim() == ''){
			doneHeaders = true;
		}
		else if (doneHeaders){
			//Console.writeln('Read: '+buffer);
			data += buffer;
		}
	}
	return data;
}

HTTPRequest.prototype.read = function(){
	if (this.socketIsEOF == false){
		var buff = this.socket.read();
		if (buff.length > 0){
			this.buffer += buff;
		}
		else {
			this.socketIsEOF = true;
		}
	}

	var pos = this.buffer.indexOf('\r\n')+2;
	if (pos == 1){
		pos = this.buffer.indexOf('\r')+1;
		if (pos == 0){
			pos = this.buffer.indexOf('\n')+1;
			if (pos == 0){
				pos = this.buffer.length;
			}
		}
	}

	var data = this.buffer.substring(0, pos);
	this.buffer = this.buffer.substring(pos);

	return data;

};

HTTPRequest.prototype.close = function(){
	this.socket.close();
};


String.prototype.trim = function(){
	return this.replace(/^\s*/, '').replace(/\s*$/, '');
}

Console.writeln('Enter a URL to download: ');
var url = Console.read().trim();

Console.writeln('Enter a save path (- for stdout): ');
var file = Console.read().trim();

var request = new HTTPRequest(url);
request.open('GET');

if (file == '-'){
	Console.writeln(request.download());
}
else {
	request.downloadToFile(file);
}

request.close();

